#ifndef IOTSTRING_H
#define IOTSTRING_H

#include "array.h"

/**
 * IoTString wraps the underlying char string.
 * @author Brian Demsky <bdemsky@uci.edu>
 * @version 1.0
 */

class IoTString {
private:
	Array<char> *array;
	IoTString() {}

	/**
	 * Builds an IoTString object around the char array.  This
	 * constructor makes a copy, so the caller is free to modify the char array.
	 */

public:
	IoTString(Array<char> *_array) : array(new Array<char>(_array)) {}
	~IoTString() {}

	/**
	 * Internal method to grab a reference to our char array.  Caller
	 * must not modify it.
	 */

	Array<char> *internalBytes() { return array; }

	/**
	 * Returns a copy of the underlying char string.
	 */

	Array<char> *getBytes() { return new Array<char>(array); }

	/**
	 * Returns the length in chars of the IoTString.
	 */

	bool equals(IoTString *str) {
		uint strlength = str->array->length();
		uint thislength = array->length();
		if (strlength != thislength)
			return false;

		int result = memcmp(str->array->internalArray(), array->internalArray(), strlength);
		return result == 0;
	}

	int length() { return array->length(); }
	friend IoTString *IoTString_shallow(Array<char> *_array);
};

IoTString *IoTString_shallow(Array<char> *_array) {
	IoTString *str = new IoTString();
	str->array = _array;
	return str;
}
#endif
