package iotcloud;

import java.nio.ByteBuffer;

/**
 * This Entry records the abort sent by a given machine.
 * @author Ali Younis <ayounis@uci.edu>
 * @version 1.0
 */


class NewKey extends Entry {
	private IoTString key;
	private long machineid;

	public NewKey(Slot slot, IoTString _key, long _machineid) {
		super(slot);
		key = _key;
		machineid = _machineid;
	}

	public long getMachineID() {
		return machineid;
	}

	public IoTString getKey() {
		return key;
	}

	public void setSlot(Slot s) {
		parentslot = s;
	}

	static Entry decode(Slot slot, ByteBuffer bb) {
		int keylength = bb.getInt();
		byte[] key = new byte[keylength];
		bb.get(key);
		long machineid = bb.getLong();

		return new NewKey(slot, IoTString.shallow(key), machineid);
	}

	public void encode(ByteBuffer bb) {
		bb.put(Entry.TypeNewKey);
		bb.putInt(key.length());
		bb.put(key.internalBytes());
		bb.putLong(machineid);
	}

	public int getSize() {
		return Long.BYTES + Byte.BYTES + Integer.BYTES + key.length();
	}

	public byte getType() {
		return Entry.TypeNewKey;
	}

	public Entry getCopy(Slot s) {
		return new NewKey(s, key, machineid);
	}
}